#! /bin/sh
# ------------------------------------------------------------------------------
# /var/install/config.d/cups-update.sh - paramater update script
#
# Creation:     2023-02-21 hbfl
# Last Update:  $Id: cups-update.sh 107758 2023-11-24 11:07:57Z hbfl $
#
# Copyright (c) 2023-2024 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ------------------------------------------------------------------------------

package_name='cups'

#. /var/install/include/eislib
. /var/install/include/configlib
. /etc/config.d/base

VERSION=3.4.0

# ----------------------------------------------------------------------------
# Set the default values for configuration
# ----------------------------------------------------------------------------
get_defaults()
{
    START_CUPS='no'

    START_CUPS_CLIENT='no'
    CUPS_CLIENT_CONNECT_TO=''

    CUPS_SERVER_WEB_CONNECT_FROM=''
    CUPS_ACTIVE_WEB_GUI_ALLWAYS='yes'
    CUPS_PRESERVE_JOB_FILES='86400'
    CUPS_PRESERVE_JOB_HISTORY='yes'  
    CUPS_LOGLEVEL='warn'
}

# ----------------------------------------------------------------------------
# Read old configuration and rename old variables
# ----------------------------------------------------------------------------
rename_old_variables()
{
    # read old values
    if [ -f /etc/config.d/${package_name} ]
    then
        /var/install/bin/backup-file --quiet ${package_name}
        . /etc/config.d/${package_name}
    fi
}

# ----------------------------------------------------------------------------
# Write config and default files
# ----------------------------------------------------------------------------
make_config_file()
{
    internal_conf_file=${1}
    {
    # ------------------------------------------------------------------------
    printgpl --conf ${package_name} '2023-11-19' 'hbfl'
    # ------------------------------------------------------------------------
    echo "# CUPS-VERSION ${VERSION}"

    # ------------------------------------------------------------------------
    printgroup 'Basic configuration'
    # ------------------------------------------------------------------------
    printvar 'START_CUPS'
    printcomment    'Use: yes or no'


    printvar 'START_CUPS_CLIENT'
    printcomment    'Use: yes or no'

    # ------------------------------------------------------------------------
    printgroup 'Connects'
    # ------------------------------------------------------------------------

    printvar 'CUPS_CLIENT_CONNECT_TO'
    printcomment    'IP of CUPS Server to connect to'

    printvar 'CUPS_SERVER_WEB_CONNECT_FROM'
    printcomment    'IP or FQDN clients to connect from'
    printcomment    'sever.example.com'
    printcomment    '192.0.2.10'
    printcomment    'sever.example.com:8631'
    printcomment    'older.server.example.com/version=1.1'
    printcomment    'older.server.example.com:8631/version=1.1' 

    printvar 'CUPS_ACTIVE_WEB_GUI_ALLWAYS'
    printcomment    'Activate the Web gui allways'
    printcomment    'Use yes or no, default: yes'   

    # ------------------------------------------------------------------------
    printgroup 'Settings'
    # ------------------------------------------------------------------------

    printvar 'CUPS_PRESERVE_JOB_FILES'
    printcomment    'Specifies whether job files (documents) are'
    printcomment    'preserved after a job is printed. If a numeric'
    printcomment    'value is specified, job files are preserved for'
    printcomment    'the indicated number of seconds after printing.'
    printcomment    'Use: yes, no, or default: is "86400" (preserve 1 day).' 

    printvar 'CUPS_PRESERVE_JOB_HISTORY'
    printcomment    'Specifies whether the job history is preserved after'
    printcomment    'a job is printed. If a numeric value is specified, the'
    printcomment    'job history is preserved for the indicated number of'
    printcomment    'seconds after printing. If "Yes", the job history is'
    printcomment    'preserved until the MaxJobs limit is reached.'
    printcomment    'Use: yes or no default: is "yes".' 

    printvar 'CUPS_LOGLEVEL'
    printcomment            'Default: warn'

    # ------------------------------------------------------------------------
    printend
    # ------------------------------------------------------------------------


    } > ${internal_conf_file}
    # Set rights
    chmod 0600 ${internal_conf_file}
    chown root ${internal_conf_file}
}

# ----------------------------------------------------------------------------
# Create the check.d file
# ----------------------------------------------------------------------------
make_check_file()
{
printgpl --check ${package_name} '2023-11-19' 'hbfl' \
>/etc/check.d/${package_name}
cat >> /etc/check.d/${package_name} <<EOF
# Variable                    OPT_VARIABLE        VARIABLE_N   VALUE
START_CUPS                    -                    -           YESNO

START_CUPS_CLIENT             !START_CUPS          -           YESNO
CUPS_CLIENT_CONNECT_TO        START_CUPS_CLIENT    -           CUPS_CLIENT_CON
CUPS_SERVER_WEB_CONNECT_FROM  START_CUPS           -           IPADDR_NETWORK
CUPS_ACTIVE_WEB_GUI_ALLWAYS   START_CUPS           -           YESNO 

CUPS_PRESERVE_JOB_FILES       START_CUPS           -           CUPS_YESNO_SECONDS_CUI 
CUPS_PRESERVE_JOB_HISTORY     START_CUPS           -           CUPS_YESNO_SECONDS_CUI
CUPS_LOGLEVEL                 START_CUPS           -           CUPS_LOGLEVEL_CUI
EOF

    # Set rights for check.d file
    chmod 0600 /etc/check.d/${package_name}
    chown root /etc/check.d/${package_name}

printgpl --check_exp ${package_name}.exp '2023-11-19' 'hbfl' \
>/etc/check.d/${package_name}.exp
cat >> /etc/check.d/${package_name}.exp <<EOF
CUPS_LOGLEVEL_CUI  = 'none|emerg|alert|crit|error|warn|notice|info|debug|debug2'
                   : 'use only: none, emerg, alert, crit, error, warn, notice, info, debug, debug2'
CUPS_CLIENT_CON    = '((RE:IPADDR)|(RE:FQDN))(:(RE:PORT))?(/version=(1.0|1.1|2.0|2.1|2.2))?'
                   : 'Invalid remote cups server value, has to be <ip or fqdn>[:port] [/version=1.0|1.1|2.0|2.1|2.2], f. e. 192.168.1.3, 192.168.3.99:631, 192.168.100.1/version=1.1, ...'
CUPS_YESNO_SECONDS_CUI = '(RE:YESNO)|(RE:NUMERIC)'
                   : 'Use yes, no or seconds to hold the jobs'
EOF

    # Set rights for check.d file
    chmod 0600 /etc/check.d/${package_name}.exp
    chown root /etc/check.d/${package_name}.exp


}

# ----------------------------------------------------------------------------
# main
# ----------------------------------------------------------------------------
main()
{
    get_defaults

    # Write default config file
    make_config_file /etc/default.d/${package_name}

    # Update from old version
    rename_old_variables

    # Write new config file
    make_config_file /etc/config.d/${package_name}

    # Write check.d file
    make_check_file

    exit 0
}

# ----------------------------------------------------------------------------
# call function main
# ----------------------------------------------------------------------------
main "${@}"

# ----------------------------------------------------------------------------
# end
# ----------------------------------------------------------------------------
