#! /bin/sh
#-------------------------------------------------------------------------------
# /var/install/config.d/cups.sh - CUPS configuration
#
# Creation:     2023-02-21 hbfl
# Last Update:  $Id: cups.sh 109169 2024-02-02 11:03:43Z hbfl $
#
# Copyright (c) 2023-2024 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#-------------------------------------------------------------------------------
#set -x
. /var/install/include/eislib

#exec 2> /tmp/cups-trace$$.log
#set -x


package_name='cups'
. /etc/config.d/base
. /etc/config.d/${package_name}


client_conf='/etc/cups/client.conf'
server_conf='/etc/cups/cupsd.conf'

VERSION=3.4.0

#---------------------------------------------------------------------------
# start
#---------------------------------------------------------------------------
cups_start()
{
    if /usr/sbin/service is-enabled ${package_name}.service
    then
        /usr/sbin/service daemon-reload
        /usr/sbin/service enable ${package_name}.path
        /usr/sbin/service start ${package_name}.path
    else
        /usr/sbin/service enable ${package_name}.service
        /usr/sbin/service enable ${package_name}.path
        /usr/sbin/service start ${package_name}.path
    fi
}

#---------------------------------------------------------------------------
# stop
#---------------------------------------------------------------------------
cups_stop()
{
    /usr/sbin/service ${quiet} stop ${package_name}.socket \
                                    ${package_name}.path \
                                    ${package_name}.cups-lpd.socket \
    /usr/sbin/service disable ${package_name}.socket \
                              ${package_name}.path \
                              ${package_name}.cups-lpd.socket \
                              ${package_name}.service
}

#---------------------------------------------------------------------------
# create client config
#---------------------------------------------------------------------------
create_client_config()
{

cat >${client_conf} <<EOF
# CUPS client configuration file (optional).

# You may use /etc/cups/client.conf (system wide)
# or ~/.cups/client.conf (per user).
# For more information see "man 5 client.conf".

ServerName ${CUPS_CLIENT_CONNECT_TO}
EOF

}

#---------------------------------------------------------------------------
# create server config
#---------------------------------------------------------------------------
create_server_config()
{

cat >${server_conf} <<EOF
#
# Configuration file for the CUPS scheduler.  See "man cupsd.conf" for a
# complete description of this file.
# 

# dont list finished jobs
PreserveJobHistory ${CUPS_PRESERVE_JOB_HISTORY} 
PreserveJobFiles ${CUPS_PRESERVE_JOB_FILES}

DefaultEncryption Never
# Log general information in error_log - change "warn" to "debug"
# for troubleshooting...
LogLevel ${CUPS_LOGLEVEL}
PageLogFormat

# Specifies the maximum size of the log files before they are rotated.  The value "0" disables log rotation.
MaxLogSize 1m

# Default error policy for printers
ErrorPolicy stop-printer

# Only listen for connections from the local machine.
Listen *:631
Listen /run/cups/cups.sock

# Show shared printers on the local network.
Browsing Yes
BrowseLocalProtocols dnssd

# Default authentication type, when authentication is required...
DefaultAuthType Basic

# Web interface setting...
WebInterface ${CUPS_ACTIVE_WEB_GUI_ALLWAYS}

# Timeout after cupsd exits if idle (applied only if cupsd runs on-demand - with -l)
IdleExitTimeout 60

# Restrict access to the server...
<Location />
  Order allow,deny
  Allow ${CUPS_SERVER_WEB_CONNECT_FROM}
</Location>

# Restrict access to the admin pages...
<Location /admin>
  Order allow,deny
  Allow ${CUPS_SERVER_WEB_CONNECT_FROM}
</Location>

# Restrict access to configuration files...
<Location /admin/conf>
  AuthType Default
  Require user @SYSTEM
  Order allow,deny
  Allow ${CUPS_SERVER_WEB_CONNECT_FROM}
</Location>

# Restrict access to log files...
<Location /admin/log>
  AuthType Default
  Require user @SYSTEM
  Order allow,deny
  Allow ${CUPS_SERVER_WEB_CONNECT_FROM}
</Location>
  
# Set the default printer/job policies...
<Policy default>
  # Job/subscription privacy...
  JobPrivateAccess default
  JobPrivateValues default
  SubscriptionPrivateAccess default
  SubscriptionPrivateValues default

  # Job-related operations must be done by the owner or an administrator...
  <Limit Create-Job Print-Job Print-URI Validate-Job>
    Order deny,allow
  </Limit>

  <Limit Send-Document Send-URI Hold-Job Release-Job Restart-Job Purge-Jobs Set-Job-Attributes Create-Job-Subscription Renew-Subscription Cancel-Subscription Get-Notifications Reprocess-Job Cancel-Current-Job Suspend-Current-Job Resume-Job Cancel-My-Jobs Close-Job CUPS-Move-Job>
    Require user @OWNER @SYSTEM
    Order deny,allow
  </Limit>

  # Require authentication for CUPS-Get-Document otherwise unauthenticated users could access print job documents:
  <Limit CUPS-Get-Document>
    AuthType Default
    Require user @OWNER @SYSTEM
    Order deny,allow
  </Limit>

  # All administration operations require an administrator to authenticate...
  <Limit CUPS-Add-Modify-Printer CUPS-Delete-Printer CUPS-Add-Modify-Class CUPS-Delete-Class CUPS-Set-Default CUPS-Get-Devices>
    AuthType Default
    Require user @SYSTEM
    Order deny,allow
  </Limit>

  # All printer operations require a printer operator to authenticate...
  <Limit Pause-Printer Resume-Printer Enable-Printer Disable-Printer Pause-Printer-After-Current-Job Hold-New-Jobs Release-Held-New-Jobs Deactivate-Printer Activate-Printer Restart-Printer Shutdown-Printer Startup-Printer Promote-Job Schedule-Job-After Cancel-Jobs CUPS-Accept-Jobs CUPS-Reject-Jobs>
    AuthType Default
    Require user @SYSTEM
    Order deny,allow
  </Limit>

  # Only the owner or an administrator can cancel or authenticate a job...
  <Limit Cancel-Job CUPS-Authenticate-Job>
    Require user @OWNER @SYSTEM
    Order deny,allow
  </Limit>

  <Limit All>
    Order deny,allow
  </Limit>
</Policy>

# Set the authenticated printer/job policies...
<Policy authenticated>
  # Job/subscription privacy...
  JobPrivateAccess default
  JobPrivateValues default
  SubscriptionPrivateAccess default
  SubscriptionPrivateValues default

  # Job-related operations must be done by the owner or an administrator...
  <Limit Create-Job Print-Job Print-URI Validate-Job>
    AuthType Default
    Order deny,allow
  </Limit>

  <Limit Send-Document Send-URI Hold-Job Release-Job Restart-Job Purge-Jobs Set-Job-Attributes Create-Job-Subscription Renew-Subscription Cancel-Subscription Get-Notifications Reprocess-Job Cancel-Current-Job Suspend-Current-Job Resume-Job Cancel-My-Jobs Close-Job CUPS-Move-Job CUPS-Get-Document>
    AuthType Default
    Require user @OWNER @SYSTEM
    Order deny,allow
  </Limit>

  # All administration operations require an administrator to authenticate...
  <Limit CUPS-Add-Modify-Printer CUPS-Delete-Printer CUPS-Add-Modify-Class CUPS-Delete-Class CUPS-Set-Default>
    AuthType Default
    Require user @SYSTEM
    Order deny,allow
  </Limit>

  # All printer operations require a printer operator to authenticate...
  <Limit Pause-Printer Resume-Printer Enable-Printer Disable-Printer Pause-Printer-After-Current-Job Hold-New-Jobs Release-Held-New-Jobs Deactivate-Printer Activate-Printer Restart-Printer Shutdown-Printer Startup-Printer Promote-Job Schedule-Job-After Cancel-Jobs CUPS-Accept-Jobs CUPS-Reject-Jobs>
    AuthType Default
    Require user @SYSTEM
    Order deny,allow
  </Limit>

  # Only the owner or an administrator can cancel or authenticate a job...
  <Limit Cancel-Job CUPS-Authenticate-Job>
    AuthType Default
    Require user @OWNER @SYSTEM
    Order deny,allow
  </Limit>

  <Limit All>
    Order deny,allow
  </Limit>
</Policy>

# The policy below is added by SUSE during build of our cups package.
# The policy 'allowallforanybody' is totally open and insecure and therefore
# it can only be used within an internal network where only trused users exist
# and where the cupsd is not accessible at all from any external host, see
# http://en.opensuse.org/SDB:CUPS_and_SANE_Firewall_settings
# Have in mind that any user who is allowed to do printer admin tasks
# can change the print queues as he likes - e.g. send copies of confidental
# print jobs from an internal network to any external destination, see
# http://en.opensuse.org/SDB:CUPS_in_a_Nutshell
# For documentation regarding 'Managing Operation Policies' see
# http://www.cups.org/documentation.php/doc-1.7/policies.html
<Policy allowallforanybody>
  # Allow anybody to access job's private values:
  JobPrivateAccess all
  # Make none of the job values to be private:
  JobPrivateValues none
  # Allow anybody to access subscription's private values:
  SubscriptionPrivateAccess all
  # Make none of the subscription values to be private:
  SubscriptionPrivateValues none
  # Allow anybody to do all IPP operations:
  # Currently the IPP operations Validate-Job Cancel-Jobs Cancel-My-Jobs Close-Job CUPS-Get-Document
  # must be additionally exlicitly specified because those IPP operations are not included
  # in the "All" wildcard value - otherwise cupsd prints error messages of the form
  # "No limit for Validate-Job defined in policy allowallforanybody and no suitable template found."
  <Limit Validate-Job Cancel-Jobs Cancel-My-Jobs Close-Job CUPS-Get-Document>
    Order deny,allow
    Allow from all
  </Limit>
  # Since CUPS > 1.5.4 the "All" wildcard value must be specified separately,
  # otherwise clients like "lpstat -p" just hang up,
  # see https://bugzilla.opensuse.org/show_bug.cgi?id=936309
  # and https://www.cups.org/str.php?L4659
  <Limit All>
    Order deny,allow
    Allow from all
  </Limit>
</Policy>
# Explicitly set the CUPS 'default' policy to be used by default:
DefaultPolicy default
EOF

}

# ---------------------------------------------------------------------------
# set parameter to no
# ---------------------------------------------------------------------------
set_parameter_no()
{
    # set parameter from 'yes' to 'no'
    sed -i "s/^${1}=.*/${1}='no'/" /etc/config.d/cups 

    # echo parameter
    echo "${1}='no'" >/tmp/eis-conf.$$
    . /tmp/eis-conf.$$
    rm -f /tmp/eis-conf.$$

    return
}

# ---------------------------------------------------------------------------
# check avahi
# ---------------------------------------------------------------------------
check_avahi()
{
    if ! /usr/sbin/service is-active avahi-daemon.service 
    then
        # set parameter from 'yes' to 'no'
        sed -i "s/^${1}=.*/${1}='yes'/" /etc/config.d/avahi

        /var/install/config.d/avahi.sh
    fi

    return
}

#---------------------------------------------------------------------------
# main
#---------------------------------------------------------------------------
main()
{
    quietmode=false

    while [ ${#} -gt 0 ]
    do
        case ${1} in
        --quiet)
            quietmode=true
            shift
            ;;
        esac
    done

    case ${START_CUPS} in
    yes)
        set_parameter_no "START_CUPS_CLIENT"
        create_server_config
        check_avahi
        cups_start
        ;;
    no)
        cups_stop
        ;;
    esac

    case ${START_CUPS_CLIENT} in 
    yes)
        create_client_config
        ;;
    no)
        cp ${client_conf}.default ${client_conf}
        ;;
    esac
}

# ---------------------------------------------------------------------------
# call function main
# ---------------------------------------------------------------------------
main "${@}"

# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
